# Generated by Django 4.0.1 on 2022-02-16 10:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0005_auto_20211116_0840'),
    ]

    operations = [
        migrations.RenameField(
            model_name='cloudinaryfile',
            old_name='basename',
            new_name='resource_name',
        ),
        migrations.RenameField(
            model_name='cloudinaryfileitem',
            old_name='basename',
            new_name='resource_name',
        ),
        migrations.RenameField(
            model_name='cloudinaryimage',
            old_name='basename',
            new_name='resource_name',
        ),
        migrations.RenameField(
            model_name='cloudinaryimageitem',
            old_name='basename',
            new_name='resource_name',
        ),
        migrations.RenameField(
            model_name='cloudinarymedia',
            old_name='basename',
            new_name='resource_name',
        ),
        migrations.RenameField(
            model_name='cloudinarymediaitem',
            old_name='basename',
            new_name='resource_name',
        ),
    ]
