# Generated by Django 3.2.6 on 2021-09-10 11:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0002_auto_20210407_0627'),
    ]

    operations = [
        migrations.DeleteModel(
            name='CloudinaryCollection',
        ),
    ]
