# Generated by Django 4.0.1 on 2022-02-17 14:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0007_alter_cloudinaryfile_resource_name_and_more'),
    ]

    run_before = [
        ('paper_uploads', '0004_remove_fileitem_created_at_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cloudinaryfileitem',
            name='created_at',
        ),
        migrations.RemoveField(
            model_name='cloudinaryfileitem',
            name='modified_at',
        ),
        migrations.RemoveField(
            model_name='cloudinaryimageitem',
            name='created_at',
        ),
        migrations.RemoveField(
            model_name='cloudinaryimageitem',
            name='modified_at',
        ),
        migrations.RemoveField(
            model_name='cloudinarymediaitem',
            name='created_at',
        ),
        migrations.RemoveField(
            model_name='cloudinarymediaitem',
            name='modified_at',
        ),
    ]
