# Generated by Django 3.1.8 on 2021-04-07 06:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='cloudinarycollection',
            options={'default_permissions': ()},
        ),
        migrations.AlterModelOptions(
            name='cloudinaryfileitem',
            options={'default_permissions': (), 'verbose_name': 'File item', 'verbose_name_plural': 'File items'},
        ),
        migrations.AlterModelOptions(
            name='cloudinaryimagecollection',
            options={'default_permissions': ()},
        ),
        migrations.AlterModelOptions(
            name='cloudinaryimageitem',
            options={'default_permissions': (), 'verbose_name': 'Image item', 'verbose_name_plural': 'Image items'},
        ),
        migrations.AlterModelOptions(
            name='cloudinarymediaitem',
            options={'default_permissions': (), 'verbose_name': 'Media item', 'verbose_name_plural': 'Media items'},
        ),
    ]
