# Generated by Django 3.2.9 on 2021-11-16 08:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0004_auto_20211106_1004'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cloudinaryfileitem',
            name='owner_app_label',
        ),
        migrations.RemoveField(
            model_name='cloudinaryfileitem',
            name='owner_fieldname',
        ),
        migrations.RemoveField(
            model_name='cloudinaryfileitem',
            name='owner_model_name',
        ),
        migrations.RemoveField(
            model_name='cloudinaryimageitem',
            name='owner_app_label',
        ),
        migrations.RemoveField(
            model_name='cloudinaryimageitem',
            name='owner_fieldname',
        ),
        migrations.RemoveField(
            model_name='cloudinaryimageitem',
            name='owner_model_name',
        ),
        migrations.RemoveField(
            model_name='cloudinarymediaitem',
            name='owner_app_label',
        ),
        migrations.RemoveField(
            model_name='cloudinarymediaitem',
            name='owner_fieldname',
        ),
        migrations.RemoveField(
            model_name='cloudinarymediaitem',
            name='owner_model_name',
        ),
    ]
